<?php
/**
 * User: nester_a
 * Date: 07.10.13
 */

namespace LMS\Tools\Conference\Base;

use LMS\Libraries\ServiceLocator;
use LMS\Tools\Conference\Abstracted;
use LMS\Libraries\Injection;
use LMS\Libraries\Exception\Operation;
use LMS\Tools\Conference\Response;
use LMS\Tools\Conference\Service;
use LMS\Tools\Conference\Repositories;

include_once __DIR__ . '/../../../includes/component.lib.php';
include_once __DIR__ . '/../../../includes/libraries/lms.lib.legacy.php';

abstract class Plugin extends Abstracted\Plugin
{

    /**
     * @var Response\Processor
     */
    protected $responseService;

    /**
     * @var Service\View
     */
    protected $viewService;

    /**
     * @var Service\File
     */
    protected $fileService;

	/**
	 * @var Service\Booking
	 */
	protected $bookingService;

	/**
	 * @var \LMS\Libraries\Date\Handler
	 */
	protected $dateHandler;

    /**
     * @param array $args
     * @return mixed
     */
    public function update(&$args)
    {
        $noNeedBeforeEvent = array(
            'onoauth'
        );

        if($args[0] instanceof Injection\Container){
            $this->initServices($args[0]);
        }

        if(isset($args[0]) && $args[0] instanceof Injection\Container && !in_array($args['event'], $noNeedBeforeEvent)){
            $this->beforeEventCalled($args[0]);
        }

        return parent::update($args);
    }


    /**
     * @param $diContainer
     */
    protected function initServices($diContainer)
    {
        $this->responseService = new Response\Processor($diContainer);
        $this->viewService = new Service\View();
        $this->fileService = new Service\File();
		$this->dateHandler = $diContainer['dateHandler'];
		$this->bookingService = new Service\Booking(array(
			'repository' => $this->getConferenceBookingRepository($diContainer['db']),
			'container' => $diContainer
		));
    }

    /**
     * @param $diContainer
     */
    protected function beforeEventCalled($diContainer)
    {
        $this->diContainer = $diContainer;
        $this->eventParams = $diContainer['registry']->get('params');
		$this->eventParams = array_merge($this->eventParams, $this->params->toArray());

        $this->responseService->setCaching($this->params->get('enableCaching'));
        $this->responseService->setTemplateDir($this->dir);
    }





    /**
     * Custom event that triggered when conferencing start page should be shown
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceStartPage($diContainer)
    {

        $courseID = $this->eventParams['id'];

        $this->responseService->addRenderParams($this->eventParams);
        $isAvailableConferenceExists = $this->bookingService->isAvailableConferenceExists($courseID, $this->eventParams['my']);

        if($isAvailableConferenceExists){
            $this->responseService->addRenderParams($this->_getParamsStartPage($this->eventParams));
			$this->responseService->addRenderParams(array(
				'config' => $diContainer['config']
			));
            return $this->responseService->renderStart();
        }

        $repository = $this->getConferenceBookingRepository($this->diContainer['db']);;

        $soonestConferences = $this->eventParams['isManager'] ?
            $repository->getTeacherSoonest($courseID, $this->eventParams['my']->id) :
            $repository->getStudentSoonest($courseID, $this->eventParams['my']->id) ;


        $this->responseService->addRenderParams(array(
            'soonestConferences' => $soonestConferences,
			'config' => $diContainer['config']
        ));

        return $this->responseService->renderStartNoAvailableConference();
    }

    /**
     * Custom event that triggered when try to enter conferencing room
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceEnterRoom($diContainer)
    {
        $courseID = $this->eventParams['courseID'];

        $isAvailableConferenceExists = $this->bookingService->isAvailableConferenceExists($courseID, $this->eventParams['my']);

        $this->responseService->addRenderParams($this->eventParams);
        if($isAvailableConferenceExists){
            $this->responseService->addRenderParams($this->_getParamsEnterRoom($this->eventParams));
            return $this->responseService->renderRoom();
        }

        $this->responseService->addRenderParams($this->_getParamsEnterRoom($this->eventParams));
        return $this->responseService->renderEnterRoomError();
    }




    /**
     *
     * @param Injection\Container $diContainer
     * @return bool|string
     */
    public function onConferenceUserAccess($diContainer)
    {
        $courseID = $this->eventParams['courseID'];
        $config = $diContainer['config'];
        $bookedConferenceID = $this->eventParams['bookedConferenceID'];

        $lists = array();

        $model = $this->getConferenceUserModel($this->diContainer['db']);;
        $ids = $model->getAllowedToAccess($bookedConferenceID);

        $userIDs = count($ids) ? implode(',', $ids) : '';

        $rows = $model->getAll($bookedConferenceID);
        $users = $model->getCourseUsersWithoutAccessToConference($courseID, $userIDs);
        $users = $this->viewService->prepareUsernames($users, $config->get('use_global_groups', 1));

        $lists['usrs'] = \mosHTML::selectList($users, 'sel_user', 'size="1" ', 'id', 'username', '' );

        $this->responseService->addRenderParams(array(
            'rows' => $rows,
            'lists' => $lists,
        ));
        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderUserAccess();
    }

    /**
     * Custom event that triggered when user requested to add(save) user in conference
     *
     * @param Injection\Container $diContainer
     * @return bool
     */
    public function onConferenceSaveUserAccess($diContainer)
    {
        $model = $this->getConferenceUserModel($this->diContainer['db']);

        if(!$model->save($this->eventParams)){
			throw new Operation\Error(
				'Error while adding user to conference',
				sefRelToAbs('index.php?option=com_joomla_lms&task=conference.user&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID'])
			);
		}

        return $this->responseService->operationOk('User has been successfully saved');
    }

    /**
     * @param $diContainer
     * @return mixed
     */
    public function onConferenceDeleteUser($diContainer)
    {
        $userIDsArray = is_array($this->eventParams['userIDs']) ? $this->eventParams['userIDs'] : array($this->eventParams['userIDs']);

        $model = $this->getConferenceUserModel($this->diContainer['db']);
        if(!$model->deleteFromConference($this->eventParams['bookedConferenceID'], $userIDsArray)){
			throw new Operation\Error(
				'Error while deleting user(s) from conference',
				sefRelToAbs('index.php?option=com_joomla_lms&task=conference.user&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID'])
			);
		}

        return $this->responseService->operationOk('User(s) has been successfully deleted');
    }





    /**
     * Custom event that triggered when request to show list of available records
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceShowArchive($diContainer)
    {
        $config = $diContainer['config'];
        $courseID = $this->eventParams['courseID'];
        $limitStart = $this->eventParams['limitStart'];
        $limit = $this->eventParams['limit'];

        $where = !$this->eventParams['isManager'] ? " AND a.published = 1 " : '';

        $model = $this->getConferenceRecordRepository($this->diContainer['db']);;
        $total = $model->count($courseID, $where);


        $this->responseService->addRenderParams(array(
            'pageNavigation' => $this->viewService->getPageNavigation( $total, $limitStart, $limit),
            'records' => $model->getAll($courseID, $limitStart, $limit),
            'liveSite' => $config->get('live_site'),
        ));
        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderArchive();
    }

    /**
     * Custom event that triggered when we start request
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceStartRecord($diContainer)
    {
        $sessionName = $this->eventParams['sessionName'];

        if(!$this->eventParams['sessionName']){
            return $this->responseService->jsonError('Session Name does not provided');
        }

        $model = $this->getConferenceRecordRepository($this->diContainer['db']);;
        $record = $model->getBySessionName($sessionName);

        $this->eventParams['user_id'] = $this->eventParams['my']->id;

        if (!$record){
            if(!$model->save($this->eventParams)){
                return $this->responseService->jsonError('Error saving record information in DB');
            }
        }

        return $this->responseService->jsonOk('Starting recording was successful');
    }

    /**
     * Custom events that triggered when request to edit record. Displays form to edit it.
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceEditRecord($diContainer)
    {
        $model = $this->getConferenceRecordRepository($this->diContainer['db']);;
        $record = $model->getByID(is_array($this->eventParams['recordID']) ? $this->eventParams['recordID'][0] : $this->eventParams['recordID']);

        if (!is_object($record)) {
			throw new Operation\Error(
				'There is no record with such ID',
				sefRelToAbs('index.php?option=com_joomla_lms&task=conference.archive&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID'])
			);
        }

        $lists = $this->viewService->prepareYesNoLists(array(
            'published' => $record->published
        ));


        $this->responseService->addRenderParams(array(
            'lists' => $lists,
            'record' => $record,
        ));
        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderRecordEdit();
    }

    /**
     * Custom event that triggered when change published status of record
     *
     * @param Injection\Container $diContainer
     */
    public function onConferenceChangeRecord($diContainer)
    {
        $courseID = $this->eventParams['courseID'];
        $this->eventParams['published'] = $this->eventParams['state'] == 1 ? $this->eventParams['state'] : 0;

        $model = $this->getConferenceRecordRepository($this->diContainer['db']);

        $where = array(
            'id' => $this->eventParams['recordID'],
            'course_id' => $courseID
        );

		$url = sefRelToAbs('index.php?option=com_joomla_lms&task=conference.archive&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID']);
        if(!$model->update($this->eventParams, $where)){
			throw new Operation\Error(
				'Error while updating record',
				$url
			);
		}

		$diContainer['router']->setBackURL($url);
        return $this->responseService->operationOk('Conference record has been successfully changed');
    }

    /**
     * Custom event that triggered when save record
     *
     * @param Injection\Container $diContainer
     */
    public function onConferenceSaveRecord($diContainer)
    {
        $courseID = $this->eventParams['courseID'];
        $model = $this->getConferenceRecordRepository($this->diContainer['db']);

        $where = array(
            'id' => $this->eventParams['recordID'],
            'course_id' => $courseID
        );

		$url = sefRelToAbs('index.php?option=com_joomla_lms&task=conference.archive&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID']);
        if(!$model->update($this->eventParams, $where)){
			throw new Operation\Error(
				'Error while saving record',
				$url
			);
		}

		$diContainer['router']->setBackUrl($url);
        return $this->responseService->operationOk('Conference record has been successfully saved');
    }

    /**
     * Custom event that triggered when request to delete record from an archive
     *
     * @param $diContainer
     * @return bool
     */
    public function onConferenceDeleteRecord($diContainer)
    {
        $courseID = $this->eventParams['courseID'];
        $recordID = $this->eventParams['recordID'];

        $recordID = is_array($recordID) ? $recordID : array(0);
        $recordIDs = implode(',', $recordID);
        require_once(_JOOMLMS_FRONT_HOME . "/includes/lms_del_operations.php");
        JLMS_deleteFromFMS($recordIDs, $courseID, $this->eventParams['option'], true);

		$url = sefRelToAbs('index.php?option=com_joomla_lms&task=conference.archive&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID']);
		$diContainer['router']->setBackUrl($url);
        return $this->responseService->operationOk('Conference record has been successfully deleted');
    }





    /**
     * Custom events that triggered when request to show list of bookings
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceShowBooking($diContainer)
    {
		$config = $diContainer['config'];
        $courseID = $this->eventParams['courseID'];
        $filterTeacher = $this->eventParams['filterTeacher'];
        $filterStudent = $this->eventParams['filterStudent'];

		$this->eventParams['history'] = false;

        $bookings = $this->bookingService->fetchData($this->eventParams);


     	/* seems like unused - need to check
     	if ($this->eventParams['doRequery']) {
            $bookings = $this->bookingService->getRequery($bookings, $diContainer['config']->get('offset'));
            $diContainer['session']->set('limitstart_confbook',  $bookings->limitStart);
        } */

        $courseModel = $this->getCourseRepository($this->diContainer['db']);
        $filters = array('teacher' => $filterTeacher, 'student' => $filterStudent);
        $lists = $this->viewService->getLists($courseModel, $courseID, $filters);

        $this->responseService->addRenderParams(array(
			'config' => $diContainer['config'],
            'pageNavigation' => $this->viewService->getPageNavigation($bookings->total, $bookings->limitStart, $bookings->limit),
            'lists' => $lists,
            'rows' => $bookings->rows,
            'userModel' => $this->getConferenceUserModel($this->diContainer['db']),
			'liveSite' => $config->get('live_site'),
        ));

        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderBooking();
    }

	/**
	 * Custom events that triggered when request to show list of bookings
	 *
	 * @param Injection\Container $diContainer
	 * @return string
	 */
	public function onConferenceShowPastBooking($diContainer)
	{
		$config = $diContainer['config'];
		$courseID = $this->eventParams['courseID'];
		$filterTeacher = $this->eventParams['filterTeacher'];
		$filterStudent = $this->eventParams['filterStudent'];

		$this->eventParams['history'] = true;
		$this->eventParams['future'] = false;

		$bookings = $this->bookingService->fetchData($this->eventParams);

		$courseModel = $this->getCourseRepository($this->diContainer['db']);
		$filters = array('teacher' => $filterTeacher, 'student' => $filterStudent);
		$lists = $this->viewService->getLists($courseModel, $courseID, $filters);

		$this->responseService->addRenderParams(array(
			'config' => $diContainer['config'],
			'pageNavigation' => $this->viewService->getPageNavigation($bookings->total, $bookings->limitStart, $bookings->limit),
			'lists' => $lists,
			'rows' => $bookings->rows,
			'userModel' => $this->getConferenceUserModel($this->diContainer['db']),
			'liveSite' => $config->get('live_site'),
		));
		$this->responseService->addRenderParams($this->eventParams);
		return $this->responseService->renderPastBooking();
	}

    /**
     * Custom event that triggered when request form for new booking conference and show this form
     *
     * @param Injection\Container $diContainer
     * @return bool|string
     */
    public function onConferenceNewBooking($diContainer)
    {
        $courseID = $this->eventParams['courseID'];

        $bookedConference = $this->bookingService->initBooking($this->eventParams['my']);
        $lists = $this->viewService->getDateTimeLists();

        $courseModel = $this->getCourseRepository($this->diContainer['db']);;
        $teachers = $courseModel->getTeachers($courseID);
        $lists['teacher_id'] = $this->viewService->prepareSelectBox($teachers, $bookedConference->teacher_id, 'teacher_id');

        $this->responseService->addRenderParams(array(
			'config' => $diContainer['config'],
            'booking' => $bookedConference,
            'lists' => $lists,
			'editor' => $this->diContainer['editor']
        ));
        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderBookingNew();
    }

    /**
     * Custom event that triggered when request to edit booking conference
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceEditBooking($diContainer)
    {
        // because we get conference ID number from the check box so it's an array
        $bookedConferenceID = array_pop($this->eventParams['bookedConferenceID']);

        $model = $this->getConferenceBookingRepository($this->diContainer['db']);;
        $bookingConference = $model->getByID($bookedConferenceID);

        if(!is_object($bookingConference)){
			throw new Operation\Error(
				'There is no such conference',
				sefRelToAbs('index.php?option=com_joomla_lms&task=conference.booking&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID'])
			);
        }

        $bookingConference->cur_date = date("Y-m-d", $bookingConference->from_time);
        $lists = $this->viewService->getDateTimeLists(array(
            'fromTime' => $bookingConference->from_time,
            'toTime' => $bookingConference->to_time,
        ));

        $courseModel = $this->getCourseRepository($this->diContainer['db']);;
        $teachers = $courseModel->getTeachers($this->eventParams['courseID']);
        $lists['teacher_id'] = $this->viewService->prepareSelectBox($teachers, $bookingConference->teacher_id, 'teacher_id');

        $this->responseService->addRenderParams(array(
            'lists' => $lists,
            'booking' => $bookingConference,
			'editor' => $this->diContainer['editor']
        ));
        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderBookingEdit();
    }

    /**
     * @param $diContainer
     * @return bool
     */
    public function onConferenceUpdateBooking($diContainer)
    {

        $bookedConferenceID = $this->eventParams['bookedConferenceID'];
        $cPublic = $this->eventParams['c_public'];

        $this->eventParams['from_time'] = $this->viewService->prepareDate($this->eventParams['from_hour'], $this->eventParams['from_min'], $this->eventParams['start_date']);
        $this->eventParams['to_time'] = $this->viewService->prepareDate($this->eventParams['to_hour'], $this->eventParams['to_min'], $this->eventParams['start_date']);

        $bookingRepository = $this->getConferenceBookingRepository($this->diContainer['db']);

		$this->bookingService->validate($this->eventParams);

        $userModel = $this->getConferenceUserModel($this->diContainer['db']);
        $cPublic = $userModel->countAllowedToAccess($bookedConferenceID) > 0 ? 0 : $cPublic;
        $this->eventParams['c_public'] = $cPublic;

        $where = array( 'p_id' => $bookedConferenceID);

		$url = sefRelToAbs('index.php?option=com_joomla_lms&task=conference.booking&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID']);
        if(!$bookingRepository->update($this->eventParams, $where)){
			throw new Operation\Error(
				'Conference for this period already exists',
				$url
			);
		}

		$diContainer['router']->setBackUrl($url);
        return $this->responseService->operationOk('Conference booking has been successfully updated');
    }

	/**
	 * Custom event that triggered when change published status of record
	 *
	 * @param Injection\Container $diContainer
	 */
	public function onConferenceChangeBooking($diContainer)
	{
		$courseID = $this->eventParams['courseID'];

		$model = new Repositories\Booking(array('db' => $diContainer['db']));

		$where = array(
			'p_id' => $this->eventParams['conferenceID'],
			'course_id' => $courseID
		);

		$url = sefRelToAbs('index.php?option=com_joomla_lms&task=conference.booking&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID']);
		if(!$model->update(array('published' => $this->eventParams['state']), $where)){
			throw new Operation\Error(
				'Error while updating booking',
				$url
			);
		}

		$diContainer['router']->setBackURL($url);
		return $this->responseService->operationOk('Conference booking has been successfully changed');
	}

    /**
     * @param $diContainer
     * @return bool
     */
    public function onConferenceSaveBooking($diContainer)
    {
        $this->eventParams['start_date'] = !empty($this->eventParams['start_date']) ? $this->eventParams['start_date'] : $this->dateHandler->getCurrentDatetime();
        $this->eventParams['from_time'] = $this->viewService->prepareDate($this->eventParams['from_hour'], $this->eventParams['from_min'], $this->eventParams['start_date']);
        $this->eventParams['to_time']  = $this->viewService->prepareDate($this->eventParams['to_hour'], $this->eventParams['to_min'], $this->eventParams['start_date']);

        $repository = $this->getConferenceBookingRepository($this->diContainer['db']);;

		$url = sefRelToAbs('index.php?option=com_joomla_lms&task=conference.booking&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID']);
        if($this->eventParams['sel_option']){
			$preparedData = $this->bookingService->prepareMultipleBookings($this->eventParams);
            $repository->saveMultiple($preparedData);
        }else{
			$this->bookingService->validate($this->eventParams);
            if(!$repository->save($this->eventParams)){
				throw new Operation\Error(
					'Error while saving conference',
					$url
				);
			}
        }
		$diContainer['router']->setBackUrl($url);
        return $this->responseService->operationOk('Conference(s) is(are) successfully saved');
    }

    /**
     * Custom event that triggered when request to delete one or more booking
     *
     * @param Injection\Container $diContainer
     * @return bool
     */
    public function onConferenceDeleteBooking($diContainer)
    {
        $bookedConferenceIDs = $this->eventParams['bookedConferenceID'];
        $bookingRepository = $this->getConferenceBookingRepository($this->diContainer['db']);;

		$url = sefRelToAbs('index.php?option=com_joomla_lms&task=conference.booking&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID']);

        $where = array(
            'p_id' => $bookedConferenceIDs
        );
        if (!$bookingRepository->delete($where)) {
			throw new Operation\Error(
				'Error while deleting conference',
				$url
			);
        }

        $userModel = $this->getConferenceUserModel($this->diContainer['db']);

        if(!$userModel->deleteAllFromBookedConferences($bookedConferenceIDs)){
			throw new Operation\Error(
				'Error while deleting all users from conference',
				$url
			);
		}


		$diContainer['router']->setBackUrl($url);
        return $this->responseService->operationOk('Conference has been successfully deleted. User from conference have been successfully deleted');
    }


	/**
	 * Custom events that triggered when request to view details for specified booked conference
	 *
	 * @param Injection\Container $diContainer
	 * @return string
	 */
	public function onConferenceViewBooking($diContainer)
	{
		$repository = $this->getConferenceBookingRepository($this->diContainer['db']);;
		$bookedConference = $repository->getByID($this->eventParams['bookedConferenceID']);

		$this->responseService->addRenderParams(array(
			'booking' => $bookedConference,
			'config' => $diContainer['config']
		));
		$this->responseService->addRenderParams($this->eventParams);
		return $this->responseService->renderBookingDetails();
	}


    /*****************************************************************************
     *
     *
     *
     *
     *
     *
     * *****************   PROTECTED METHODS TO BE OVERWRITTEN   ***************
     *
     *
     *
     *
     *
     *
     *
     **************************************************************************/




    protected function getConferenceBookingRepository($db)
    {
        return new Repositories\Booking(array('db' => $db));
    }

    protected function getConferenceRecordRepository($db)
    {
        return new Repositories\Record(array('db' => $db));
    }

    protected function getConferenceUserModel($db)
    {
        return new Repositories\User(array('db' => $db));
    }

    protected function getCourseRepository($db)
    {
        return new Repositories\Course(array('db' => $db));
    }

    protected function getConferenceDocModel($db)
    {
        return new Repositories\Doc(array('db' => $db));
    }

}